///////////////////////////////////////////////////////////////////////////////////////////////
//----------------------------------------Client32.cpp---------------------------------------//
//																							 //
// This is the 32-bit rendition of the Client.cpp console program. As with the original		 //
// Client.cpp program, this app will connect to the Server to exchange data, but this time   //
// a file will be sent, not just a simple message.											 //
//																							 //
// Created by: Drew Sikora and Mike Mahoney													 //
// Created on: 10.15.00																		 //
//																							 //
//-------------------------------------------Notes-------------------------------------------//
//																							 //
// One distinct note would be that the term Server here is used as the endpoint for the data //
// being sent. It's not really a server in that it'll collect data and then send it to other //
// clients. It really is a client, but to offer up a distinction between the sending program //
// and the receiving program, we've opted to call them Client and Server, respectively.		 //
//																							 //
///////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////
//------------------------------------------Includes-----------------------------------------//

// MFC is the devil, exorcise it
#define WIN32_LEAN_AND_MEAN

// Windows includes
#include <windows.h>	// the main Windows header
#include <windowsx.h>	// extra Windows functions

// WinSock includes
#include <winsock2.h>	// the WindSock v2 header file
#include <winsockx.h>	// extra WindSock functions

// other includes
#include <colors.h>	// set of predefined COLORREF RGB colors
#include <stdio.h>		// needed for using sprintf()
#include <string.h>		// needed for using strcat()
#include <fstream.h>	// needed for file I/O
#include <memory.h>		//
#include <malloc.h>		// used for memory allocation and manipulation

//-------------------------------------------Defines-----------------------------------------//

// window defines
#define WINDOW_WIDTH	640			// in pixels
#define WINDOW_HEIGHT	580			// in pixels
#define WINDOW_NAME		"Client"	// window class name

// WinSock versions
#define WS_VERSION1	0x0101	// WinSock v1.1
#define WS_VERSION2 0x0202	// WinSock v2.0

// IP defines
#define IP_ADDR		"216.87.134.224"	// the server IP address
#define PORT_NUM	5000				// the port number

// app states
#define END		0	// the app has ended (sockets closed and not sending data)
#define RUN		1	// the app is still running (sockets still open and sending data)
#define RESTART	2	// the user wants to send file again

// end of file states
#define EOFT	0	// the end of the file is reached
#define NOTEOF	1	// the file still has more to go

// packet defines
#define MAX_PACKET_SIZE	500						// the max packet size in kilobytes

// LineText flag defines
#define HOLD		0	// the text does not move
#define ADVANCE		1	// text moves down one line
#define HADVANCE	2	// used to advance text right after HOLD flag is used

// how long till connect() times out (20 seconds)
#define TIME_OUT	5000

//-------------------------------------------Structures--------------------------------------//

// the first-sent struct heralding the info on the coming data
struct HEADER
{
	LPSTR	filename;	// the name of the file being sent
	LPSTR	location;	// where to create the file
	int		size;		// the size of the file (in bytes)
};

// the actual packet being sent
struct PACKET
{
	int		index;		// the packet index number, used for order confirmation
	LPSTR	data;		// the data read from the file
	int		file;		// tells whether the end of the file is reached
};

//-------------------------------------------Instances---------------------------------------//

// program instances
MSG			msg;		// generic message holder
HDC			gdc;		// handle to the graphic device context
HWND		hwnd;		// main window handle (used to ID app in Windows)
RECT		rect;		// rectangle coords struct, used to validate window
fstream		in;			// define Input file I/O
HINSTANCE	ginstance;	// global window instance (used to track app in Windows)

// Winsock instances
SOCKET		Port;			// the data socket
WSAData		stWSAData;		// holds WinSock data (i.e. version, max socket #, etc.)
sockaddr_in	Server;			// address struct pointing to the server

// struct instances
HEADER	header;	// packet header
PACKET	packet;	// main packet data

//---------------------------------------------Styles----------------------------------------//

// class styles
UINT CSTYLE = CS_OWNDC|		// the window has its own DC handle assigned to it
			  CS_DBLCLKS|	// the window can recognize double mouse clicks
			  CS_HREDRAW|	// the window auto-redraws after any horizontal resize
			  CS_VREDRAW;	// same as above, except only for vertical resize

// window styles
DWORD WSTYLE = WS_OVERLAPPEDWINDOW|	// a window with all the trimmings
			   WS_VISIBLE;			// make the window visible

//--------------------------------------------Globals----------------------------------------//

int		app_state;	// the execution state of the app
int		line_spot;	// the line position to draw new text
DWORD	tick_time;	// the time that the file transfer was started (as measured by the system
					// clock, in milliseconds)
int		pack_index;	// the index counter for the packets
int		data_sent;	// total amount of data sent (in bytes), note this is not accurate

//-------------------------------------------Prototypes--------------------------------------//

int Main();
int StartUp();
int CleanUp();
int LineText(HWND hwnd, LPSTR buffer, COLORREF color, int flags);
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

///////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------WinMain()--------------------------------------//
//																							 //
// WinMain is where it all begins and where it all ends. All other functions branch off and  //
// return to this one. When this function terminates, the entire program terminates.		 //
//																							 //
//--------------------------------------------Variables--------------------------------------//
//																							 //
// hinstance		- the current instance that is used by Windows to track our app			 //
// hprevhinstance	- a field that was used back in the days of Win 3.1 but not today		 //
// lpcmdline		- a null-terminating string that supplies any command line parameters	 //
// ncmdshow			- an integer that indicates how the main app window is to be opened 	 //
//																							 //							 
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow)
{
	// create the window class
	WNDCLASSEX	winclass;

	// fill in the class struct
	winclass.cbSize			= sizeof(WNDCLASSEX);				// size of the struct
	winclass.style			= CSTYLE;							// class style flags
	winclass.lpfnWndProc	= WindowProc;						// the callback function
	winclass.cbClsExtra		= 0;								// extra param (unused)
	winclass.cbWndExtra		= 0;								// extra param (unused)
	winclass.hInstance		= hinstance;						// handle to app instance
	winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);	// icon in titlebar (default)
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);		// window cursor (default)
	winclass.hbrBackground	= GetStockObject(BLACK_BRUSH);		// paint the window black
	winclass.lpszMenuName	= NULL;								// no menu resource loaded
	winclass.lpszClassName	= WINDOW_NAME;						// name of class
	winclass.hIconSm		= LoadIcon(NULL, IDI_APPLICATION);	// icon in taskbar (default)

	// register the cass with Windows
	if (!RegisterClassEx(&winclass))
	{
		// display an error message and clean up
		MessageBox(hwnd, "Could not register class!", "", MB_OK | MB_ICONEXCLAMATION);
	}

	// create the window
	if (!(hwnd = CreateWindowEx(NULL,			// extra creation param (unused)
								WINDOW_NAME,	// class to derive info from
								"Client",		// text in titlebar
								WSTYLE,			// window style flags
								0, 0,			// initial x, y coords
								WINDOW_WIDTH,	// width of window (in pixels)
								WINDOW_HEIGHT,	// height of window (in pixels)
								NULL, NULL,		// no menu or parent window present
								hinstance,		// app instance handle to attach to
								NULL)))			// extra creation param (unused)
	{
		// display an error message and clean up
		MessageBox(hwnd, "Could not create window!", "", MB_OK | MB_ICONEXCLAMATION);
		return(0);
	}

	// load up WinSock and connect to the Server
	StartUp();
								
	///////////////////////////
	// enter main event loop //
	///////////////////////////

	while(TRUE)
	{
		// test for any messages in the queue
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			// test for a quit message
			if (msg.message == WM_QUIT)
				break;	// exit the loop

			// translate any accelerator keys
			TranslateMessage(&msg);

			// send the message off to the callback function
			DispatchMessage(&msg);
		}
	
		// test whether or not the program has ended
		if (app_state != END)
		{
			// send the file (or continue sending the file)
			Main();
		}

	}

	//////////////////////////
	// exit main event loop //
	//////////////////////////

	// return control to Windows
	return(msg.wParam);

} // end WinMain


///////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------WindowProc()----------------------------------------//
//																							 //
// This is the program's callback function, or where all the windows messages (they start    //
// with WM_) are processed. Any message sent by windows like WM_TIMER or WM_PAINT to the	 //
// main program are foisted off onto this function to be dealt with accordingly.			 //
//																							 //
//-----------------------------------------Variables-----------------------------------------//
//																							 //
// hwnd		- the handle of the window the messages are from. Only really matters when you 	 //
//			  are dealing with more than one window.										 //
// msg		- the structure that contains the actual message type along with any extra para- //
//			  meters.																		 //
// wparam	- am extra message parameter													 //
// lparam	- an extra message parameter													 //
//																							 //
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)/////////////////
{
	// use a switch statement to process the message types
	switch(msg)
	{
	case WM_PAINT:		// the window needs repainting for some reason
		{
			// get the DC from Windows
			gdc = GetDC(hwnd);

				// if the app is restarting, clear out the window
				if (app_state == RESTART)
				{
					// get the area of the window
					GetClientRect(hwnd, &rect);

					// fill it in with black
					FillRect(gdc, &rect, CreateSolidBrush(BLACK));
				}
				
			// release the DC back to Windows
			ReleaseDC(hwnd, gdc);

			// validate the window so we stop getting Paint messages
			GetClientRect(hwnd, &rect);	// get the coords of the client area and store in rect
			ValidateRect(hwnd, &rect);	// validate the given rect (entire client area)

			// return success
			return(0);
		} break;
	case WM_KEYDOWN:	// detects a keypress
		{
			// extract the keycode
			int key_code = (int)wparam;

			// sift through key_code options
			switch(key_code)
			{
			case VK_ESCAPE:	
				{
					// verify the user's request
					int answer = MessageBox(hwnd, "Are you sure you want to quit?", "", 
											MB_YESNO | MB_ICONEXCLAMATION);

					// if the user says OK...
					if (answer == IDYES)
						// ... exit the program
						PostMessage(hwnd, WM_DESTROY, 0, 0);
				} break;
			case VK_RETURN:
				{
					// if the program still runs, ignore this command
					if (app_state == RUN)
						break;

					// set the state to RESTART and repaint the window
					app_state = RESTART;
					SendMessage(hwnd, WM_PAINT, 0, 0);

					// call the StartUp() function
					StartUp();
				} break;
			default: break;
			}

			// return succes
			return(0);
		} break;
	case WM_DESTROY:	// the window has been closed, but the app is still running
		{ 
			// if the program is still running, then cleanup
			if (app_state == RUN)
				CleanUp();

			// post a quit message so that the program terminates
			PostQuitMessage(0);

			// return success
			return(0);
		} break;
	default: break;
	}

	// any messages we didn't handle we can let Windows deal with
	return(DefWindowProc(hwnd, msg, wparam, lparam));

} // end WindowProc

///////////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------StartUp()--------------------------------------//
//																							 //
// This function will load WinSock and set up all the ports and sockets to allow the sending //
// of data to the Server.																	 //
//																							 //
int StartUp()//////////////////////////////////////////////////////////////////////////////////
{
	// set the program state to RUN
	app_state = RUN;
	
	// reset variables
	line_spot = 
	data_sent = 
	tick_time = 0;

	// update the user
	LineText(hwnd, "Loading WinSock...", RED, ADVANCE);

	// load the WinSock DLL
	if (WSAStartup(WS_VERSION2, &stWSAData))
	{
		// display an error message and clean up
		LineText(hwnd, "Could not load WinSock!", BLUE, ADVANCE);
		CleanUp();
		return(0);
	}

	// update the user
	LineText(hwnd, "Creating socket...", RED, ADVANCE);

	// create and test the socket
	Port = socket(AF_INET, SOCK_STREAM, 0);
	if (Port == INVALID_SOCKET)
	{
		// display an error message and clean up
		LineText(hwnd, "Could not create socket!", BLUE, ADVANCE);
		CleanUp();
		return(0);
	}

	// update the user
	LineText(hwnd, "Socket created successfully.", RED, ADVANCE);
	LineText(hwnd, "Waiting to connect to Server...", RED, ADVANCE);

	// fill out the socket info struct
	Server.sin_family		= AF_INET;				// address internet family
	Server.sin_port			= htons(PORT_NUM);		// port number (must match Servers)
	Server.sin_addr.s_addr	= inet_addr(IP_ADDR);	// Server IP address

	// get the connect loop start time
	tick_time = GetTickCount();

	// connect to the server on Port
	int result;
	result = connect(Port, (LPSOCKADDR)&Server, sizeof(Server));
	if (result)
	{
		// display an error message and clean up
		LineText(hwnd, "Could not connect to Server. Connection may have timed out.",
				 BLUE, ADVANCE);
		CleanUp();
		return(0);
	}
			
	// update the user
	LineText(hwnd, "Connected to Server!", RED, ADVANCE);			
	LineText(hwnd, "Opening file for read...", RED, ADVANCE);

	// holds the filename and path
	LPSTR fname = "C:\\";
	LPSTR path	= "Test.txt";  

	// create the full file path
	strcat(path, fname);

	// open the file if it exists
	in.open(path, ios::binary|ios::nocreate);  
												
	// update the user
	LineText(hwnd, "File has been opened successfully, getting data...", RED, ADVANCE);

	// fill out the HEADER struct 
	header.filename	= fname;
	header.location	= path;
	header.size		= 0;

	// get the size of the header
	int header_size = sizeof(HEADER);

	// create a buffer to hold the header info
	char *lpBuff = new char[header_size];

	// copy in the header info to the buffer
	memcpy(lpBuff, &header, header_size);  

	// send the packet header
	LineText(hwnd, "Sending packet header...", RED, ADVANCE);
	if (send(Port, lpBuff, header_size, 0) == SOCKET_ERROR)
	{
		// display an error message and clean up
		LineText(hwnd, "Could not send packet header!", BLUE, ADVANCE);
		CleanUp();
		return(0);
	}

	// update the user
	LineText(hwnd, "Packet header sent! Beginning file transfer", RED, ADVANCE);

	// get the starting time of the transfer
	tick_time = GetTickCount();

	// return success
	return(0);

} // end StartUp

///////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------Main()----------------------------------------//
//																							 //
// This function has the duty of reading more and more data from the file and then sending   //
// it off to the Server. This function alone decides when the program state turns to END,	 //
// that happens when the end of the file is reached.										 //
//																							 //
int Main()/////////////////////////////////////////////////////////////////////////////////////
{
	// increment and assign the packet index
	pack_index++;
	packet.index = pack_index;

	// read in more data from the file and store it in packet.data. 
	in.read((LPSTR)&packet.data, sizeof(MAX_PACKET_SIZE));

	// get the size of the packet struct
	int packet_size = sizeof(PACKET);

	// create a buffer to store the packet data in
	char *lpBuff = new char[packet_size];

	// copy the data from packet into the buffer
	memcpy(lpBuff, &packet, packet_size);  

	// send the packet off to the Server
	if (send(Port, lpBuff, packet_size, 0) == SOCKET_ERROR)
	{
		// display an error message and clean up
		LineText(hwnd, "Could not send packet!", BLUE, ADVANCE);
		CleanUp();
		return(0);
	}

	// increment the amount of data sent
	data_sent += MAX_PACKET_SIZE;

	// update the user on what was sent
	char sent[50];
	sprintf(sent, "%i bytes sent out of %i bytes.", data_sent, header.size);
	LineText(hwnd, sent, RED, HOLD);

	// decide whether the program has ended
	if (packet.file == EOFT)
		CleanUp();

	// return success
	return(0);

} // end Main

///////////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------CleanUp()--------------------------------------//
//																							 //
// This function will unload WinSock after it closes all the ports and sockets and dis-		 //
// connects from the server.																 //
//																							 //
int CleanUp()//////////////////////////////////////////////////////////////////////////////////
{
	// define time holder variable
	int time = 0;

	// get the total alotted time that the transfer took
	if (tick_time != 0)
		time = ((GetTickCount() - tick_time));

	// shutdown the socket and tell the user
	shutdown(Port, 2);
	LineText(hwnd, "Socket has been disconnected", RED, ADVANCE);

	// close the socket and tell the user
	closesocket(Port);
	LineText(hwnd, "Socket has been shutdown", RED, ADVANCE);

	// cleanup by unloading the WinSock DLL and informing the user
	WSACleanup();
	LineText(hwnd, "WinSock has been unloaded.", RED, ADVANCE);

	// close the file
	in.close();
	LineText(hwnd, "File has been closed.", RED, ADVANCE);

	// display the time it took to transfer the file
	LPSTR strTime;
	sprintf(strTime, "Time of file transfer: %i", time);
	LineText(hwnd, strTime, RED, ADVANCE);

	// let the user restart the program by pressing Enter
	LineText(hwnd, "To test again, press Enter. Or press Escape to quit.", RED, ADVANCE);

	// set the program state to END
	app_state = END;

	// return success
	return(0);

} // end CleanUp


///////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------LineText()--------------------------------------//
//																							 //
// This program will space text for the program - every 15 pixels is a new line of text. It  //
// can also hold the text to one line so that a counter can increment in realtime. However,  //
// while the HOLD flag is set, no other text can be written.								 //
//																							 //
//-------------------------------------------Variables---------------------------------------//
//																							 //
// hwnd		- the window handle																 //
// buffer	- the message to output															 //
// color	- text color																	 //
// flags	- behavior flags, hold a line or advance to the next							 //
//																							 //
int LineText(HWND hwnd, LPSTR buffer, COLORREF color, int flags)///////////////////////////////
{
	// define the number of pixels between each line
	#define LINE 15

	// define variables for text drawing
	int			old_bkmode;
	COLORREF	old_tcolor;

	// define the static variable to hold the state of a line (hold or not)
	// 1 = no hold
	// 0 = hold
	static int hold = 1;

	// get the DC from Windows so we can write text onscreen
	gdc = GetDC(hwnd);

	// set the text color and the background mode (transparent) and store the old 
	// values so we can reset them later on.
	old_tcolor = SetTextColor(gdc, color);
	old_bkmode = SetBkMode(gdc, TRANSPARENT);

	if (flags == HOLD)
	{
		// set the hold variable to HOLD
		hold = HOLD;

		// get the client area in case the window was resized horizontally
		GetClientRect(hwnd, &rect);

		// reset the other values to narrow the rect to just the line of text to erase
		rect.left	= 0;
		rect.top	= line_spot + 3;
		rect.bottom	= line_spot + LINE;

		// erase the text with a solid black rectangle
		FillRect(gdc, &rect, CreateSolidBrush(BLACK));
	}

	// decide whether to turn off hold
	if ((flags == ADVANCE) && (hold == HOLD))
	{
		// turn off hold and advance
		hold = 1;
		line_spot += LINE;
	}

	// print out the text
	TextOut(gdc, 0, line_spot, buffer, strlen(buffer));

	// reset the old values
	SetTextColor(gdc, old_tcolor);
	SetBkMode(gdc, old_bkmode);

	// release the DC back to Windows
	ReleaseDC(hwnd, gdc);

	// advance the line
	if (flags == ADVANCE)
	line_spot += LINE;
	
	// return success
	return(0);

} // end LineText

///////////////////////////////////////////////////////////////////////////////////////////////
//-----------------------------------------Client32.cpp--------------------------------------//
//--------------------------------Copyright Napali Networks, 2000----------------------------//
///////////////////////////////////////////////////////////////////////////////////////////////