///////////////////////////////////////////////////////////////////////////////////////////////
//-----------------------------------------Packet.h------------------------------------------//
//																							 //
// The Packet class is used to send data encapsulated in the regular IP header. The Packet	 //
// class is set up to allow sending and receiving of files, and text messages. The Packet	 //
// class works through the Socket class to send and receive its data. The packet class is    //
// responsible for tracking, ordering, and interpreting all incoming and outgoing data.		 //
//																							 //
// Created by: Drew Sikora																	 //
// Created on: 11/12/00																		 //
//																							 //
//------------------------------------Functionality Notes------------------------------------//
//																							 //
// The Packet class will not directly send data. The main program will use the Socket class  //
// to send and receive. It is the job of the packet class to make sure all packets reach the //
// destination intanct, and to keep them in the order they were sent so the program can re-	 //
// assemble them. Note that the Packet class only packages the data, it does not create it.  //
// So if the packets were bits of data from a file, the main program would extract the bits  //
// and then fill out the Packet class file structure. The packet class is not responsible    //
// for opening the file, reading in data, etc.												 //
//																							 //
///////////////////////////////////////////////////////////////////////////////////////////////

class Packet
{
public:

	// constructors
	// the constructors differ for text messages and file data (order: file, text)
	Packet();
	Packet();

	// member functions
	void FillHeader(char filename, char location);
	void FillPacket(HEADER header, LPSTR data, int file);
	PARRAY OrderPackets(PARRAY parray);

	// deconstructor
	~Packet();

protected:

	// member functions
	int FileSize();	// gets the size of the file

	// member variables
	int index;	// packet index number (current)

	// member structures
	// packet header
	struct HEADER
	{
		char	filename[50];	// the name of the file
		char	location[100];	// the files destination location
		int		size;			// the size of the file (in bytes)
	};

	// main packet
	struct PACKET
	{
		HEADER	header;	// the packet header struct
		int		index;	// the index number of the packet (sent order)
		LPSTR	data;	// the data (bytes) sent with the packet
		int		file;	// whether the end of the file has been reached (all data sent)
	};

	// packet array.
	struct PARRAY
	{
		PACKET packet;
	};

};
