///////////////////////////////////////////////////////////////////////////////////////////////
//-----------------------------------------Socket.h------------------------------------------//
//																							 //
// The Socket class is defined here for use in conjunction with Socket.cpp. The socket class //
// can be used for any program dealing with asynchronous windows sockets. Theoretically, it  //
// could be used for any WinSock program, but it is optomized for async sockets with multi-	 //
// ple connections and using it for blocking sockets may cause errore.						 //
//																							 //
// Created by: Drew Sikora																	 //
// Created on: 11/12/00																		 //
//																							 //
//------------------------------------Functionality Notes------------------------------------//
//																							 //
// The Socket class must be able to handle any sort of operation required from a socket as   //
// well as be able to report the status of the socket at any time during execution if the	 //
// program asks it. The main functions it must support:										 //
//																							 //
// accept()		- The Socket class must be able to accept any incoming connection requests 	 //
//				  and create a new socket to take on the new connection. This new socket	 //
//				  will inherit the Socket functionality.									 //
// listen()		- The Socket must be able to listen for a client connection.				 //
// connect()	- The Socket class has to be able to request a connection to a remote server //
//				  and be prepared to handle the results (does connect or errors out.		 //
// recv()		- The Socket class should be able to receive asyncronous data and store it	 //
//				  in the assigned buffer.													 //
// send()		- The Socket class needs to be able to send data if so required. Data will	 //
//				  be extracted from the assigned buffer.									 //
// closesocket()- The Socket class must be able to close a given socket at anytime and grace-//
//				  fully handle any errors that may result from an imprompt closing.			 //
// shutdown()	- The Socket class must be able to perform graceful closes on sockets after  //
//				  it has determined there is no data left to send.							 //
//																							 //
// The other functions that the class must support are strictly maintenance only. They are   //
// such things as being able to check the amount of data sent, received, the time taken so	 //
// far, how much time is left before a connection times out.								 //
//																							 //
///////////////////////////////////////////////////////////////////////////////////////////////

class Socket
{
public:

	// constructors
	// client and server constructors differ (order: client, server)
	Socket(LPSTR IPAddr, int PortNum, int Type, int Protocol, long Events, UINT Msg);	
	Socket(int IPAddr, int PortNum, int Type, int Protocol, long Events, UINT Msg);		
	
	// Normal WinSock member functions
	int Close();		// the closesocket() handler
	int Connect();		// the connect() handler
	int SendData();		// the send() handler
	int RecvData();		// the recv() handler
	int ShutDown();		// the shutdown() handler
	SOCKET Accept();	// the accept() handler
	SOCKET Listen();	// the listen() handler

	// Packet WinSock member functions
	int SendData();		// used for sending file Packets
	PACKET RecvData();	// used for receving file Packets

	// maintenenance member functions
	int TimeLeft();			// returns amount of time left until connection (or listen) attempt		
							// times out (in seconds).
	SOCKDATA SockData();	// returns socket data information

	// destructor
	~Socket();
	
protected:

	// WinSock member variables/instances
	SOCKET		socket;		// main socket (client/server)
	SOCKET		listen;		// listening socket (server)
	sockaddr	sock_addr;	// socket properties struct (client/server)
	sockaddr_in	sock_in;	// listening socket properties struct (server)

	// maintenance member functions
	int DataSent();			// returns total bytes sent
	int DataRecv();			// returns total bytes received
	int TransferTime();		// returns total time taken to transfer data file (in milliseconds)

	// socket info struct
	struct SOCKDATA
	{
		LPSTR IPAddr;	// the IP address of the socket
		int Protocol;	// the socket protocol name
		int Type;		// the socket protocol type
		int PortNum;	// socket port number
		int DataSent;	// number of bytes sent (total)
		int DataRecv;	// number of bytes received (total)
		int Connect;	// total time of connection (in milliseconds)
	};
		
};
